	org $0800-2
	dw  $0800
	
	include "music/!playerv.asm"

rand			= $70

joy_left_pressed	= $80
joy_right_pressed	= $81
joy_up_pressed		= $82
joy_down_pressed	= $83
joy_fire_pressed	= $84
;
key_left_pressed	= $80
key_right_pressed	= $81
key_up_pressed		= $82
key_down_pressed	= $83
key_fire_pressed	= $84
;
repeat_delay		= $8E
repeat_count		= $8F

player_x		= $90		; logical position: 0=left, 4=right
player_screen_x		= $91		; screen character position
player_color		= $92		; 0-4
shot_x			= $93
shot_color		= $94
shot_y			= $95		; 0=not shooting
shot_animation_index	= $96

gem_y			= $A0		; 5 bytes
gem_color		= $A5		; 5 bytes

game_speed		= $B0
game_tick		= $B1
speedup_tick		= $B2
game_status		= $B3		; 0=playing, otherwise game over
score			= $B4		; 5 bytes
high_score		= $B9		; 5 bytes
score_add		= $BE		; 2 bytes
last_gem_color		= $C0
bonus_count		= $C1		; when 4 is reached, speed decreases

dst_ptr			= $C2		; generic pointers
col_ptr			= $C4
gem_src			= $C6
gem_current_color	= $C8

ldst			= $D0		; lightning pointers
lsrc1			= $D2
lsrc2			= $D4
lsrc3			= $D6
lsrc4			= $D8
lvalue			= $DA

; keyboard
keyb_ptr		= $E0		; 16 bit
keyb_lastkey		= $E2
keyb_repeatcnt		= $E3
keyb_shift		= $E4
keyb_repeat_init	= 20		; constant
keyb_repeat		= 4		; constant

main_menu_index		= $E5
title_flash_index	= $E6

MAIN_MENU_SCREEN_POS	= $0C00+40*13+12
MAIN_MENU_COLOR		= $46
MAIN_MENU_SELECTED	= $53

REPEAT_DELAY		= 12
INIT_GAME_SPEED		= 35
FASTEST_GAME_SPEED	= 10
SLOW_DOWN_VALUE		= 1
SPEED_INCREASE		= 8

	include "screen_hex.asm"
	include "font.asm"
	
start
	sei
	sta $ff3f
	;
	jsr init_video			; one-time init
	;
	jsr init_music
	;
	jsr InitRandom
	;
	;jsr init_screen
	;
	jsr reset_player
	;
	lda #$00
	ldx #$04
il1	sta score,x
	sta high_score,x
	dex
	bpl il1
	;
	lda #$2
	sta lvalue
	;
	lda #$00
	sta main_menu_index
	sta title_flash_index
	;
	lda #$1B
	sta $FF06
	bne title_start_skip_wipe	; always jump
	;
title_start
	jsr reset_gems
	jsr erase_player
	jsr wipe_screen
	;
	lda score
	ora score+1
	ora score+2
	ora score+3
	ora score+4
	beq no_new_hs
	;
	ldx #$00
chs1	lda score,x
	cmp high_score,x
	beq chs2
	bcc no_new_hs
	bcs new_hs
chs2	inx
	cpx #$05
	bne chs1
	;
new_hs	ldx #$04
chs5	lda score,x
	sta high_score,x
	lda #$00
	sta score,x
	dex
	bpl chs5
	;
	ldx #19
chs6	lda txt_congrats,x
	sta $0EDA,x
	lda #$51
	sta $0ADA,x
	dex
	bpl chs6
	;
no_new_hs
	lda high_score
	ora high_score+1
	ora high_score+2
	ora high_score+3
	ora high_score+4
	beq title_start_skip_wipe
	;
	ldx #15
dhs1	lda txt_high_score,x
	sta $0F04,x
	lda #$51
	sta $0B04,x
	dex
	bpl dhs1
	;
	ldx #$02			; default: print 3 digits (skip first two)
	lda high_score
	ora high_score+1
	beq *+4
	ldx #$00			; print all if score is >=1000
dhs2	lda high_score,x
	ora #$30
	sta $0F0F,x
	inx
	cpx #$05
	bne dhs2
	;
title_start_skip_wipe
	jsr display_title
	;
	jsr main_menu_select
	;
	lda #$00
	sta player_color
	sta game_speed
	jsr update_right_panel
	jsr update_speed 
	;
	jsr draw_lightning_chars
	;
menu_loop				; main menu    
	lda #main_menu_key_table & 255
	ldx #main_menu_key_table >> 8
	jmp keyboard

game_start
	jsr wipe_screen
	;
	jsr init_controls
	;
	jsr init_game
	;
game_loop				; game loop
	jsr sync
	;
	;inc $FF19
	jsr animate_shot
	jsr update_speed
	;
	jsr move_shot
	jsr move_gems
	;
	lda game_status
	bne game_over_loop
	;
control_call
	jsr check_joy
	;
	jsr update_player
	;
	jsr write_score
	jsr draw_lightning
	;dec $FF19
	;
	jmp game_loop
	
game_over_loop
	lda #100			; roughly two second wait
gol1	pha				; (to avoid user pressing fire too quickly)
	jsr sync
	jsr draw_lightning
	jsr flash_gem
	pla
	sec
	sbc #$01
	bne gol1
	;
	jsr wait_fire_or_space
	;
	jmp title_start
	
title_flash
	ldx #$03
tf0	ldy title_flash_index
	cpy #$03
	bcc skip_title_flash
	cpy #$18
	bcs skip_title_flash
tf1	lda title_flash_colors,x
	sta $0800+40*05+9,y
	sta $0800+40*06+8,y
	sta $0800+40*07+7,y
	dey
	dex
	bpl tf1
	;
skip_title_flash
	ldy title_flash_index
	cpy #$17
	bcc skip_twinkle
	cpy #$17+10
	bcs skip_twinkle
	;
	tya
	sec
	sbc #$17
	asl a
	asl a
	asl a
	asl a
	clc
	adc #twinkle & 255
	sta dst_ptr
	lda #twinkle >> 8
	adc #$00
	sta dst_ptr+1
	;
	ldy #$0F
tf2	lda (dst_ptr),y
	sta font+$BD*8,y
	dey
	bpl tf2
	;
skip_twinkle
	inc title_flash_index
	;
	rts
	
main_menu_frame
	jsr draw_lightning_safe
	jsr title_flash
	rts
	
main_menu_deselect
	ldx main_menu_index
	lda main_menu_screen_pos_lo,x
	sta dst_ptr
	sta col_ptr
	lda main_menu_screen_pos_hi,x
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	ldy #$00
	lda #$20
	sta (dst_ptr),y
	;
	ldy #15
	sta (dst_ptr),y
	;
	lda #MAIN_MENU_COLOR
mmds1	sta (col_ptr),y
	dey
	bpl mmds1
	rts

main_menu_select
	ldx main_menu_index
	lda main_menu_screen_pos_lo,x
	sta dst_ptr
	sta col_ptr
	lda main_menu_screen_pos_hi,x
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	ldy #$00
	lda #$3E
	sta (dst_ptr),y
	;
	ldy #15 
	lda #$3C
	sta (dst_ptr),y
	;
	lda #MAIN_MENU_SELECTED
mms1	sta (col_ptr),y
	dey
	bpl mms1
	rts
	
main_menu_up
	lda main_menu_index
	beq mmu1
	;
	jsr main_menu_deselect
	dec main_menu_index
	jsr main_menu_select
	;
mmu1	jmp menu_loop

main_menu_down
	lda main_menu_index
	cmp #$02
	beq mmd1
	;
	jsr main_menu_deselect
	inc main_menu_index
	jsr main_menu_select
	;
mmd1	jmp menu_loop

main_menu_select_joy
	ldx #check_joy & 255
	ldy #check_joy >> 8
	bne mmsk1
main_menu_select_key
	ldx #check_key & 255
	ldy #check_key >> 8
mmsk1	stx control_call+1
	sty control_call+2
	;
	lda main_menu_index
	beq mms2
	;
	cmp #$01
	beq instructions
	;
	jsr wipe_screen			; credits
	jsr display_credits
	jsr wait_fire_or_space
	jmp title_start
	;
mms2	jmp game_start

instructions
	jsr wipe_screen
	jsr display_instructions
	jsr wait_fire_or_space
	jmp title_start

sync
	lda $FF1D			; sync to $C8-$CF range (due to digi, we may skip some rasterlines)
	and #$F8
	cmp #$C8
	bne sync
	rts
	
init_music
	lda #$00			; self-mod
	jsr PLAYER_INIT
	;
	; ----------
	; 
	lda #$AA
	sta $FF0A
	;
	lda #irq & 255
	sta $FFFE
	lda #irq >> 8
	sta $FFFF
	;
	lda #$30
	sta $FF0B
	;
	lda #$00
	sta $FF00
	lda #$F0
	sta $FF01 
	;
	cli
	rts

irq
	;dec $FF19
	pha
	txa
	pha
	tya
	pha
	;
	lda $FF09
	sta $FF09
	tax
	and #$08
	beq no_timer
	;
digi_index
	lda #$00
	clc
	adc #$01
	cmp #$05
	bne *+4
	lda #$00
	sta digi_index+1
	clc
	adc VOLUME
	sta $FF11
	;
no_timer
	txa
	and #$02
	beq no_raster
	;
	jsr PLAYER
	;
	lda $FF0B
	eor #($CC ^ $30)
	sta $FF0B
	;
no_raster
	pla
	tay
	pla
	tax
	pla
	;inc $FF19
	rti
	
init_controls
	lda #$00
	sta joy_left_pressed
	sta joy_right_pressed
	sta joy_up_pressed
	sta joy_down_pressed
	sta joy_fire_pressed
	rts
	
wait_fire_or_space
	jsr sync
	jsr flash_gem
	jsr draw_lightning
	;
	jsr check_fire
	bne wfos1
	;
	jsr check_space
	bne wait_fire_or_space
	;
wfos2	jsr sync
	jsr flash_gem
	jsr draw_lightning
	;
	jsr check_space
	beq wfos2
	rts
	;
wfos1	jsr sync
	jsr flash_gem
	jsr draw_lightning
	;
	jsr check_fire
	beq wfos1
	rts

check_fire
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	and #$C0			; fire
	cmp #$C0
	rts
	
check_space
	lda #$7F			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	and #$10			; space
	rts

	;
	; ---
	;	
check_joy
	jsr check_joy_left
	jsr check_joy_right
	jsr check_joy_up
	jsr check_joy_down
	jsr check_joy_fire
	rts
	
check_joy_left
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	;
	ldx joy_left_pressed
	bne check_joy_left_release
	;
	and #$04			; left
	bne cjl_1
	;
cjl_3	lda #REPEAT_DELAY
	sta joy_left_pressed		; flag that left is in "pressed" state
	;
	lda player_x
	beq cjl_4
	;
	dec player_x
cjl_4	rts
	;
check_joy_left_release
	and #$04			; left
	bne cjl_2
	;
	dec joy_left_pressed
	rts
	;
cjl_2	lda #$00
	sta joy_left_pressed
	;
cjl_1	rts

check_joy_right
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	;
	ldx joy_right_pressed
	bne check_joy_right_release
	;
	and #$08			; right
	bne cjr_1
	;
cjr_3	lda #REPEAT_DELAY
	sta joy_right_pressed		; flag that right is in "pressed" state
	;
	lda player_x
	cmp #$04
	beq cjr_4
	;
	inc player_x
cjr_4	rts
	;
check_joy_right_release
	and #$08			; right
	bne cjr_2
	;
	dec joy_right_pressed
	rts
	;
cjr_2	lda #$00
	sta joy_right_pressed
	;
cjr_1	rts

check_joy_up
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	;
	ldx joy_up_pressed
	bne check_joy_up_release
	;
	and #$01			; up
	bne cju_1
	;
cju_3	lda #REPEAT_DELAY
	sta joy_up_pressed		; flag that up is in "pressed" state
	;
	lda player_color		; toggle player color (-1)
	sec
	sbc #$01
	bpl *+4
	lda #$04
	sta player_color
	jsr update_right_panel
	;
	rts
	;
check_joy_up_release
	and #$01			; up
	bne cju_2
	;
	dec joy_up_pressed
	rts
	;
cju_2	lda #$00
	sta joy_up_pressed
	;
cju_1	rts

check_joy_down
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	;
	ldx joy_down_pressed
	bne check_joy_down_release
	;
	and #$02			; down
	bne cjd_1
	;
cjd_3	lda #REPEAT_DELAY
	sta joy_down_pressed		; flag that down is in "pressed" state
	;
	lda player_color		; toggle player color (+1)
	clc
	adc #$01
	cmp #$05
	bne *+4
	lda #$00
	sta player_color
	jsr update_right_panel
	;
	rts
	;
check_joy_down_release
	and #$02			; down
	bne cjd_2
	;
	dec joy_down_pressed
	rts
	;
cjd_2	lda #$00
	sta joy_down_pressed
	;
cjd_1	rts

check_joy_fire
	lda #$FF			; joy only
	sta $FD30
	lda #$F9			; either port 
	sta $FF08
	lda $FF08
	;
	ldx joy_fire_pressed
	bne check_joy_fire_release
	;
	and #$C0			; fire
	cmp #$C0
	beq cjf_1
	;
	lda shot_y
	bne cjf_1			; shot_y is disabled
	;
	lda #REPEAT_DELAY
	sta joy_fire_pressed		; flag that fire is in "pressed" state
	;
	lda #21
	sta shot_y
	lda player_color
	sta shot_color
	lda player_x
	sta shot_x
	;
	jsr move_shot			; draw (or not, if it immediately hits a gem)
	;
	rts
	;
check_joy_fire_release
	and #$C0			; fire
	cmp #$C0
	beq cjf_2
	;
	dec joy_fire_pressed
	rts
	;
cjf_2	lda #$00
	sta joy_fire_pressed
	;
cjf_1	rts

	;
	; ---
	;
check_key
	jsr check_key_left
	jsr check_key_right
	jsr check_key_up
	jsr check_key_down
	jsr check_key_fire
	rts
	
check_key_left
	lda #$BF			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	;
	ldx key_left_pressed
	bne check_key_left_release
	;
	and #$01			; left
	bne ckl_1
	;
ckl_3	lda #REPEAT_DELAY
	sta key_left_pressed		; flag that left is in "pressed" state
	;
	lda player_x
	beq ckl_4
	;
	dec player_x
ckl_4	rts
	;
check_key_left_release
	and #$01			; left
	bne ckl_2
	;
	dec key_left_pressed
	rts
	;
ckl_2	lda #$00
	sta key_left_pressed
	;
ckl_1	rts

check_key_right
	lda #$BF			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	;
	ldx key_right_pressed
	bne check_key_right_release
	;
	and #$08			; right
	bne ckr_1
	;
ckr_3	lda #REPEAT_DELAY
	sta key_right_pressed		; flag that right is in "pressed" state
	;
	lda player_x
	cmp #$04
	beq ckr_4
	;
	inc player_x
ckr_4	rts
	;
check_key_right_release
	and #$08			; right
	bne ckr_2
	;
	dec key_right_pressed
	rts
	;
ckr_2	lda #$00
	sta key_right_pressed
	;
ckr_1	rts

check_key_up
	lda #$DF			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	;
	ldx key_up_pressed
	bne check_key_up_release
	;
	and #$08			; up
	bne cku_1
	;
cku_3	lda #REPEAT_DELAY
	sta key_up_pressed		; flag that up is in "pressed" state
	;
	lda player_color		; toggle player color (-1)
	sec
	sbc #$01
	bpl *+4
	lda #$04
	sta player_color
	jsr update_right_panel
	;
	rts
	;
check_key_up_release
	and #$08			; up
	bne cku_2
	;
	dec key_up_pressed
	rts
	;
cku_2	lda #$00
	sta key_up_pressed
	;
cku_1	rts

check_key_down
	lda #$DF			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	;
	ldx key_down_pressed
	bne check_key_down_release
	;
	and #$01			; down
	bne ckd_1
	;
ckd_3	lda #REPEAT_DELAY
	sta key_down_pressed		; flag that down is in "pressed" state
	;
	lda player_color		; toggle player color (+1)
	clc
	adc #$01
	cmp #$05
	bne *+4
	lda #$00
	sta player_color
	jsr update_right_panel
	;
	rts
	;
check_key_down_release
	and #$01			; down
	bne ckd_2
	;
	dec key_down_pressed
	rts
	;
ckd_2	lda #$00
	sta key_down_pressed
	;
ckd_1	rts

check_key_fire
	lda #$7F			; keyboard latch
	sta $FD30
	lda #$FF			; keyboard only
	sta $FF08
	lda $FF08
	;
	ldx key_fire_pressed
	bne check_key_fire_release
	;
	and #$10			; space
	bne ckf_1
	;
	lda shot_y
	bne ckf_1			; shot_y is disabled
	;
	lda #REPEAT_DELAY
	sta key_fire_pressed		; flag that fire is in "pressed" state
	;
	lda #21
	sta shot_y
	lda player_color
	sta shot_color
	lda player_x
	sta shot_x
	;
	jsr move_shot			; draw (or not, if it immediately hits a gem)
	;
	rts
	;
check_key_fire_release
	and #$10			; space
	bne ckf_2
	;
	dec key_fire_pressed
	rts
	;
ckf_2	lda #$00
	sta key_fire_pressed
	;
ckf_1	rts

erase_shot
	ldy shot_y
	ldx shot_x
	lda gem_screen_pos_1_lo,y
	clc
	adc times_5_plus_1,x
	sta dst_ptr
	sta col_ptr
	lda gem_screen_pos_1_hi,y
	adc #$00
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	ldy #$00
	lda #$20
	sta (dst_ptr),y
	iny
	sta (dst_ptr),y
	rts

draw_shot
	ldy shot_y
	ldx shot_x
	lda gem_screen_pos_1_lo,y
	clc
	adc times_5_plus_1,x
	sta dst_ptr
	sta col_ptr
	lda gem_screen_pos_1_hi,y
	adc #$00
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	ldy #$00
	lda #$7E
	sta (dst_ptr),y
	iny
	lda #$7F
	sta (dst_ptr),y
	;
	ldx shot_color
	lda gem_colors,x
	sta (col_ptr),y
	dey
	sta (col_ptr),y
	rts
	
animate_shot
	lda #shot_animation & 255
	clc
	adc shot_animation_index
	sta dst_ptr
	lda #shot_animation >> 8
	adc #$00
	sta dst_ptr+1
	;
	ldy #$0F
as1	lda (dst_ptr),y
	sta font+$7E*8,y
	dey
	bpl as1
	;
	lda shot_animation_index
	clc
	adc #$10
	cmp #$C0
	bne *+4
	lda #$00
	sta shot_animation_index
	;
	rts
	
move_shot
	lda shot_y
	beq ms_end			; zero means there's no shot
	;
	cmp #21				; 21 is "off screen"
	beq skip_erase
	;
	jsr erase_shot
	;
skip_erase
	dec shot_y
	ldx shot_x
	lda gem_y,x
	clc
	adc #$01
	cmp shot_y
	beq gem_shot
	jsr draw_shot
	;
ms_end	rts

gem_shot
	lda #$00
	sta shot_y			; "remove" shot
	;
	ldx shot_x
	lda shot_color
	cmp gem_color,x			; check if it's the right color
	beq right_color
	;
	rts				; nothing else to do...
	;
right_color	
	jsr erase_gem			; yes, this gem is destroyed
	;
	jsr add_score
	;
	lda #$00			; reset gem to top
	ldx shot_x
	sta gem_y,x
	;
	jsr get_random_color
	sta gem_color,x
	jsr draw_gem
	;
	rts
	
move_gems
	dec game_tick			; tick down
	lda game_tick			; on zero, gems move
	bne mg_end
	;
	inc speedup_tick		; tick up
	lda speedup_tick
	cmp #SPEED_INCREASE		; after this many "steps", speed increases
	bne mg1
	;
	lda #$00
	sta speedup_tick
	;
	lda game_speed
	cmp #FASTEST_GAME_SPEED		; absolute minimum (should never be reached)
	beq mg1
	dec game_speed
	;
mg1	lda game_speed
	sta game_tick
	;
	ldx #$00
mg2	jsr erase_gem
	;
	lda gem_y,x
	clc
	adc #$01
	sta gem_y,x
	;
	jsr draw_gem
	;
	lda gem_y,x
	cmp #20
	beq game_over
	;
	cpx shot_x
	bne mg3
	;
	clc
	adc #$01
	cmp shot_y
	beq gem_shot
	;
mg3	inx
	cpx #$05
	bne mg2
	;
mg_end	rts

game_over
	ldx #$03
go1	lda txt_game_over,x
	sta $0C00+40*12+15,x
	lda txt_game_over+4,x
	sta $0C00+40*12+15+6,x
	lda #$67
	sta $0800+40*12+15,x
	sta $0800+40*12+15+6,x
	dex
	bpl go1
	;
	lda #$FF
	sta game_status
	rts
	
update_speed
	lda game_speed
	lsr a
	lsr a
	lsr a
	lsr a
	tax
	lda to_hex,x
	sta $0E82
	lda game_speed
	and #$0F
	tax
	lda to_hex,x
	sta $0E83
	;
	lda #$01
	sta $0A82
	sta $0A83
	rts
	
add_score
	lda last_gem_color
	cmp shot_color			; same as last color?
	beq ads1
	;
	lda #$00			; nope, reset added score
	sta score_add
	sta score_add+1
	sta bonus_count
	;
ads1	jsr inc_score_add
	;
	jsr add_score_ones
	jsr add_score_tens
	;
	lda shot_color			; keep track of last gem's color
	sta last_gem_color
	rts
	
inc_score_add
	inc bonus_count
	lda bonus_count
	cmp #$04			; 4 reached?
	bne isa2
	;
	lda #$00			; reset
	sta bonus_count
	;
	lda game_speed			; slow down
	clc
	adc #SLOW_DOWN_VALUE
	cmp #INIT_GAME_SPEED		; but don't slow down beyond initial speed
	bcc isa3
	lda #INIT_GAME_SPEED
isa3	sta game_speed
	lda #$00
	sta speedup_tick		; also reset the speed up tick 
	;
isa2	lda score_add+1
	clc
	adc #$01
	cmp #$0A
	bcs isa1
	sta score_add+1
	rts
isa1	sbc #$0A
	sta score_add+1
	inc score_add			; should never reach "99"
	rts

add_score_tens
	ldx #$03
	lda score_add
	beq ast1
	bne aso0
	;
add_score_ones
	ldx #$04
	lda score_add+1
	;
aso0	clc
	adc score,x
	cmp #$0A
	bcs aso1
	sta score,x			; digit did not overflow, we're done
	rts
	;
aso1	sbc #$0A
	sta score,x
	;
	lda #$01			; add 1 to previous digit
	dex
	bpl aso0			; stop at first digit, entire score overflowed
	;
	lda #$09
	ldx #$04
aso2	sta score,x
	dex
	bpl aso2
ast1	rts

init_video
	ldx #$FF
	lda $FF07
	and #$40
	beq *+4
	ldx #$F8
	cpx $FF1D
	bne *-3
	;
	lda #$00
	sta $FF15
	sta $FF19
	;
	lda $FF12
	and #$03
	sta $FF12
	lda #font >> 8
	sta $FF13
	;
	lda #$71
	sta $FF16
	lda #$31
	sta $FF17
	;
	lda $FF07
	and #$40
	ora #$98
	sta $FF07
	;
	rts
	
display_title
	ldx #15
dt2	lda txt_title,x
	sta $0C00+40*05+12,x
	lda txt_title+16,x
	sta $0C00+40*06+12,x
	lda txt_title+32,x
	sta $0C00+40*07+12,x
	;
	lda #$66+8
	sta $0800+40*05+12,x
	sta $0800+40*06+12,x
	sta $0800+40*07+12,x
	dex
	bpl dt2
	;
	ldx #11
dt3	lda txt_menu,x
	sta MAIN_MENU_SCREEN_POS+0*40+2,x
	lda txt_menu+12,x
	sta MAIN_MENU_SCREEN_POS+1*40+2,x
	lda txt_menu+24,x
	sta MAIN_MENU_SCREEN_POS+2*40+2,x
	lda #MAIN_MENU_COLOR
	sta MAIN_MENU_SCREEN_POS-$400+0*40+2,x
	sta MAIN_MENU_SCREEN_POS-$400+1*40+2,x
	sta MAIN_MENU_SCREEN_POS-$400+2*40+2,x
	dex
	bpl dt3
	;
	rts
	
wipe_screen
	lda #$07
	sta dst_ptr
	lda #$0C
	sta dst_ptr+1
	;
	ldx #20				; number of lines to clear
ws2	;
	jsr sync
	jsr draw_lightning_safe
	;
	ldy #(40-14-1)
	lda #$20
ws1	sta (dst_ptr),y
	dey
	bpl ws1
	;
	lda dst_ptr
	clc
	adc #$28
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	dex
	bpl ws2
	rts
	
display_instructions
	lda #txt_instructions & 255
	sta gem_src
	lda #txt_instructions >> 8
	sta gem_src+1
	bne display_fullscreen_text
	;
display_credits
	lda #txt_credits & 255
	sta gem_src
	lda #txt_credits >> 8
	sta gem_src+1
	;
display_fullscreen_text
	lda #$06
	sta dst_ptr
	sta col_ptr
	lda #$0C
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	ldx #0				; row counter
di2	ldy #$00
	lda (gem_src),y
	iny
	cmp #$01			; skip empty lines (indicated by $01 color)
	beq di3
	;
di4	sta player_color
	;
	jsr sync
	jsr draw_lightning_safe
	;
	ldy #$01
di1	lda (gem_src),y			; print one line
	sta (dst_ptr),y
	lda player_color
	sta (col_ptr),y
	iny
	cpy #(40-14)+1
	bne di1
	;
	lda player_color
	cmp #$50
	bcs di3
	;
	clc
	adc #$10
	bne di4
	;
di3	lda dst_ptr
	clc
	adc #$28
	sta dst_ptr
	sta col_ptr
	bcc *+6
	inc dst_ptr+1
	inc col_ptr+1
	;
	tya				; line length (+1 byte for color)
	clc
	adc gem_src
	sta gem_src
	bcc *+4
	inc gem_src+1
	;
	inx
	cpx #21				; number of lines to print
	bne di2
	rts
	
;init_screen
;	ldx #$00
;is1	lda #$56+8			; clear screen
;	sta $0800,x
;	sta $0900,x
;	sta $0A00,x
;	sta $0B00,x
;	;
;	lda #$20
;	sta $0C00,x
;	sta $0D00,x
;	sta $0E00,x
;	sta $0F00,x
;	inx
;	bne is1
;	;
;	; ---
;	;
;	lda #$00			; draw left panel
;	sta dst_ptr
;	sta col_ptr
;	lda #$0C
;	sta dst_ptr+1
;	and #$FB
;	sta col_ptr+1
;	;
;	lda #panel_left & 255
;	sta gem_src
;	lda #panel_left >> 8
;	sta gem_src+1
;	;
;	ldx #24
;is3	ldy #$000
;is2	lda (gem_src),y
;	sta (dst_ptr),y
;	lda #$59
;	sta (col_ptr),y
;	iny
;	cpy #$07
;	bne is2
;	;
;	lda dst_ptr
;	clc
;	adc #40
;	sta dst_ptr
;	sta col_ptr
;	bcc *+6
;	inc dst_ptr+1
;	inc col_ptr+1
;	;
;	lda gem_src
;	clc
;	adc #$07
;	sta gem_src
;	bcc *+4
;	inc gem_src+1
;	;
;	dex
;	bpl is3
;	;
;	; ---
;	;
;	lda #$21			; draw right panel
;	sta dst_ptr
;	sta col_ptr
;	lda #$0C
;	sta dst_ptr+1
;	and #$FB
;	sta col_ptr+1
;	;
;	lda #panel_right & 255
;	sta gem_src
;	lda #panel_right >> 8
;	sta gem_src+1
;	;
;	;
;	ldx #24
;is5	ldy #$00
;is4	lda (gem_src),y
;	sta (dst_ptr),y
;	lda #$59
;	sta (col_ptr),y
;	iny
;	cpy #$07
;	bne is4
;	;
;	lda dst_ptr
;	clc
;	adc #40
;	sta dst_ptr
;	sta col_ptr
;	bcc *+6
;	inc dst_ptr+1
;	inc col_ptr+1
;	;
;	lda gem_src
;	clc
;	adc #$07
;	sta gem_src
;	bcc *+4
;	inc gem_src+1
;	;
;	dex
;	bpl is5
;	;
;	ldy #$04			; draw score characters
;is7	lda #$35			; score color
;	sta $084A,y
;	sta $0872,y
;	;
;	lda #$57+8			; "warning" yellow color
;	sta $0A2A,y
;	sta $0919,y
;	;
;	lda #$14			; top/left "grid" thing
;	sta $0829,y
;	lda #$24
;	sta $0851,y
;	lda #$34
;	sta $0879,y
;	lda #$44
;	sta $08A1,y
;	;
;	dey
;	bpl is7
;	;
;	; ---
;	;
;	;
;	lda #$42+8			; lightning red sides
;	sta $0B4E
;	sta $0B69
;	;
;	; ---
;	;
draw_lightning_chars
	ldy #(40-14-1)			; draw lightning
is6	tya
	clc
	adc #$E0
	sta $0F4F,y
	lda #$65+8
	sta $0B4F,y
	dey
	bpl is6
	;
	rts

get_random_color
	jsr GetRandom
	and #$07
	cmp #$05
	bcs get_random_color
	rts
	
reset_gems
	lda #$00
	ldx #$04
rg1	sta gem_y,x
	dex
	bpl rg1
	rts
	
reset_player
	ldx #$02
	stx player_x
	lda player_screen_pos_lo,x
	sta player_screen_x
	lda #$00
	sta player_color
	rts

init_game
	jsr reset_player
	jsr draw_player
	jsr update_right_panel
	;
	lda #$00
	sta shot_x
	sta shot_y
	sta shot_animation_index
	;
	jsr reset_gems
	;
	ldx #$00
ig1	jsr get_random_color
	sta gem_color,x
	jsr draw_gem
	inx
	cpx #$05
	bne ig1
	;
	lda #INIT_GAME_SPEED		; inital game speed
	sta game_speed
	sta game_tick
	lda #$00
	sta speedup_tick
	;
	lda #$00
	sta game_status
	;
	sta score
	sta score+1
	sta score+2
	sta score+3
	sta score+4
	jsr write_score
	;
	lda #$00
	sta score_add
	lda #$01
	sta score_add+1
	;
	lda #$FF
	sta last_gem_color
	lda #$00
	sta bonus_count
	;
	rts
	
update_right_panel
	lda player_color
	clc
	adc #$03
	tax
	;
	lda #$EB
	sta dst_ptr
	lda #$08
	sta dst_ptr+1
	;
urp2	lda gem_colors,x
	;
	ldy #$02
urp1	sta (dst_ptr),y
	dey
	bpl urp1
	;
	lda dst_ptr
	clc
	adc #$28
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	inx
	lda dst_ptr
	cmp #$B3
	bne urp2
	;
	rts
	
update_player
	jsr erase_player
	;
	lda player_screen_x
	ldx player_x
	cmp player_screen_pos_lo,x
	beq up1
	bcs up2
	inc player_screen_x
	db $2C
up2	dec player_screen_x 
	;
up1	jsr draw_player
	rts
	
erase_player
	ldx player_screen_x
	;
	ldy #5
	lda #$20
ep1	sta $0F98,x
	sta $0FC0,x
	inx
	dey
	bpl ep1
	rts
	
draw_player
	ldx player_screen_x
	;
	ldy player_color
	lda gem_colors,y
	sta player_color_byte+1
	;
	ldy #$00
dp1	lda player_ship_chars_1,y
	sta $0F98,x
	lda player_ship_chars_2,y
	sta $0FC0,x
player_color_byte
	lda #$59			; player's ship color
	sta $0B98,x
	sta $0BC0,x
	inx
	iny
	cpy #$06
	bne dp1
	;
	lda shot_y			; is the shot moving?
	bne dp2				; yes, skip
	;
	lda #$7E			; draw "shot" in middle of ship
	sta $0FC0-4,x
	lda #$7F
	sta $0FC0-3,x
	;
dp2	rts

draw_lightning_safe
	txa
	pha
	tya
	pha
	;
	jsr draw_lightning
	;
	pla
	tay
	pla
	tax
	rts

draw_lightning
	lda #(font + $E0 * 8) & 255
	sta ldst
	lda #(font + $E0 * 8) >> 8
	sta ldst+1
	;
lightning_char_count
	ldx #26				; 40-14, number of chars to process
	;
dl1	; ---
	ldy lvalue
	lda $FF1E
	eor $FF00
	;
	cmp #$65
	bcc al30
	cmp #$9b
	bcc al31
	cpy #$04
	beq al31
	iny
	bne al31
al30	tya
	beq al31
	dey
al31	; ---
	;
	tya
	clc
	adc #lightning_1 & 255
	sta lsrc1
	lda #lightning_1 >> 8
	sta lsrc1+1
	;
	; ---
	lda $FF1E
	eor $FF00
	;
	cmp #$65
	bcc al00
	cmp #$9b
	bcc al01
	cpy #$04
	beq al01
	iny
	bne al01
al00	tya
	beq al01
	dey
al01	; ---
	;
	tya
	clc
	adc #lightning_2 & 255
	sta lsrc2
	lda #lightning_2 >> 8
	sta lsrc2+1
	;
	; ---
	lda $FF1E
	eor $FF00
	;
	cmp #$65
	bcc al10
	cmp #$9b
	bcc al11
	cpy #$04
	beq al11
	iny
	bne al11
al10	tya
	beq al11
	dey
al11	; ---
	;
	tya
	clc
	adc #lightning_3 & 255
	sta lsrc3
	lda #lightning_3 >> 8
	sta lsrc3+1
	;
	; ---
	lda $FF1E
	eor $FF00
	;
	cmp #$65
	bcc al20
	cmp #$9b
	bcc al21
	cpy #$04
	beq al21
	iny
	bne al21
al20	tya
	beq al21
	dey
al21	; ---
	;
	tya
	clc
	adc #lightning_4 & 255
	sta lsrc4
	lda #lightning_4 >> 8
	sta lsrc4+1
	;
	sty lvalue
	;
	ldy #$00
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	iny
	lda (lsrc1),y
	ora (lsrc2),y
	ora (lsrc3),y
	ora (lsrc4),y
	sta (ldst),y
	;
	lda ldst
	clc
	adc #$08
	sta ldst
	bcc *+4
	inc ldst+1
	;
	dex
	beq dl_end
	jmp dl1
dl_end	lda $FF07
	and #$40
	beq dl_end1
	;
	lda #13
	sta lightning_char_count+1
	lda draw_lightning+1
	eor #(13*8)
	sta draw_lightning+1
	;
dl_end1	rts

	; x = gem index
erase_gem
	lda gem_y,x
	tay
	lda gem_screen_pos_1_lo,y
	clc
	adc times_5,x
	sta dst_ptr
	lda gem_screen_pos_1_hi,y
	adc #$00
	sta dst_ptr+1
	;
	ldy #$03
	lda #$20
eg1	sta (dst_ptr),y			; erase characters only
	dey
	bpl eg1
	;
	lda dst_ptr
	clc
	adc #$28
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	ldy #$03
	lda #$20
eg2	sta (dst_ptr),y			; erase characters only
	dey
	bpl eg2
	rts
	
	; x = gem index
draw_gem
	lda gem_y,x
	tay
	lda gem_screen_pos_1_lo,y
	clc
	adc times_5,x
	sta dst_ptr
	sta col_ptr
	lda gem_screen_pos_1_hi,y
	adc #$00
	sta dst_ptr+1
	and #$FB
	sta col_ptr+1
	;
	lda gem_color,x
	tay
	lda gem_colors,y
	sta gem_current_color
	;
	lda gem_chars_lo,y
	sta gem_src
	lda gem_chars_hi,y
	sta gem_src+1
	;
	ldy #$03
dg11	lda (gem_src),y
	sta (dst_ptr),y
	lda gem_current_color
	sta (col_ptr),y
	dey
	bpl dg11
	;
	lda dst_ptr
	clc
	adc #40
	sta dst_ptr
	sta col_ptr
	bcc *+6
	inc dst_ptr+1
	inc col_ptr+1
	;
	lda gem_src
	clc
	adc #$04
	sta gem_src
	bcc *+4
	inc gem_src+1
	;
	ldy #$03
dg12	lda (gem_src),y
	sta (dst_ptr),y
	lda gem_current_color
	sta (col_ptr),y
	dey
	bpl dg12
	;
	rts
	
flash_gem
	ldx #$00
fg13	lda gem_y,x
	cmp #20
	beq fg10
	inx
	cpx #$05
	bne fg13
	beq fg14
	;
fg10	tay
	lda gem_screen_pos_1_lo,y
	clc
	adc times_5,x
	sta col_ptr
	lda gem_screen_pos_1_hi,y
	adc #$00
	and #$FB
	sta col_ptr+1
	;
	lda gem_color,x
	tay
	lda gem_colors,y
	and #$0F
gem_flash_index
	ldx #$00
	ora gem_flash,x
	sta gem_current_color
	;
	ldy #$03
	lda gem_current_color
fg11	sta (col_ptr),y
	dey
	bpl fg11
	;
	lda col_ptr
	clc
	adc #40
	sta col_ptr
	bcc *+4
	inc col_ptr+1
	;
	ldy #$03
	lda gem_current_color
fg12	sta (col_ptr),y
	dey
	bpl fg12
	;
	lda gem_flash_index+1
	clc
	adc #$01
	and #$07
	sta gem_flash_index+1
	;
fg14	rts
	
write_score
	lda score
	ora score+1
	beq write_score_big
	;
	ldx #$04
wrs1	lda score,x
	ora #$30
	sta $0C4A,x
	lda #$20
	sta $0C72,x
	dex
	bpl wrs1
	rts
	;
write_score_big
	ldx #$04
wsb1	txa
	ora #$C0
	sta $0C4A,x			; print the characters that hold the large digits
	clc
	adc #$05
	sta $0C72,x
	dex
	bpl wsb1
	;
	ldx score+2
	jsr write_digit_0
	ldx score+3
	jsr write_digit_1
	ldx score+4
	jsr write_digit_2
	rts
	
	; x = digit
write_digit_0
	lda #(digits0 & 255)
	clc
	adc times_32_lo,x
	sta gem_src
	lda #(digits0 >> 8)
	adc times_32_hi,x
	sta gem_src+1
	;
	lda #(font + $C0 * 8) & 255
	sta dst_ptr
	lda #(font + $C0 * 8) >> 8
	sta dst_ptr+1
	;
	ldy #$0F
wd01	lda (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd01
	;
	lda gem_src
	clc
	adc #$10
	sta gem_src
	bcc *+4
	inc gem_src+1
	;
	lda dst_ptr
	clc
	adc #5*8
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	ldy #$0F
wd02	lda (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd02
	;
	rts
	
	; x = digit
write_digit_1
	lda #(digits1 & 255)
	clc
	adc times_48_lo,x
	sta gem_src
	lda #(digits1 >> 8)
	adc times_48_hi,x
	sta gem_src+1
	;
	lda #(font + $C1 * 8) & 255
	sta dst_ptr
	lda #(font + $C1 * 8) >> 8
	sta dst_ptr+1
	;
	ldy #$17
wd11	lda (gem_src),y
	sta (dst_ptr),y
	dey
	cpy #$07
	bne wd11
	;
wd12	lda (dst_ptr),y
	ora (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd12
	;
	lda gem_src
	clc
	adc #$18
	sta gem_src
	bcc *+4
	inc gem_src+1
	;
	lda dst_ptr
	clc
	adc #5*8
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	ldy #$17
wd13	lda (gem_src),y
	sta (dst_ptr),y
	dey
	cpy #$07
	bne wd13
	;
wd14	lda (dst_ptr),y
	ora (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd14
	;
	rts
	
	; x = digit
write_digit_2
	lda #(digits2 & 255)
	clc
	adc times_32_lo,x
	sta gem_src
	lda #(digits2 >> 8)
	adc times_32_hi,x
	sta gem_src+1
	;
	lda #(font + $C3 * 8) & 255
	sta dst_ptr
	lda #(font + $C3 * 8) >> 8
	sta dst_ptr+1
	;
	ldy #$0F
wd21	lda (gem_src),y
	sta (dst_ptr),y
	dey
	cpy #$07
	bne wd21
	;
wd22	lda (dst_ptr),y
	ora (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd22
	;
	lda gem_src
	clc
	adc #$10
	sta gem_src
	bcc *+4
	inc gem_src+1
	;
	lda dst_ptr
	clc
	adc #5*8
	sta dst_ptr
	bcc *+4
	inc dst_ptr+1
	;
	ldy #$0F
wd23	lda (gem_src),y
	sta (dst_ptr),y
	dey
	cpy #$07
	bne wd23
	;
wd24	lda (dst_ptr),y
	ora (gem_src),y
	sta (dst_ptr),y
	dey
	bpl wd24
	;
	rts
	
	include "random.asm"
	
	include "key_constants.asm"
	include "keyboard.asm"
	
main_menu_key_table
	dw main_menu_frame
	;
	dw $4004, main_menu_select_joy
	dw $0104, main_menu_up
	dw $0204, main_menu_down
	dw $8002, main_menu_select_joy
	dw $0102, main_menu_up
	dw $0202, main_menu_down
	;
	dw KEY_CURSOR_DOWN, main_menu_down
	dw KEY_CURSOR_UP, main_menu_up
	dw KEY_RETURN, main_menu_select_key
	dw KEY_SPACE, main_menu_select_key
	;
	dw 0
	
player_screen_pos_lo
	db 7
	db 12
	db 17
	db 22
	db 27
	
player_ship_chars_1
	db $54,$55,$56,$57,$58,$59
player_ship_chars_2
	db $74,$75,$76,$77,$78,$79

gem_colors
	db $42+8
	db $56+8
	db $67+8
	db $43+8
	db $55+8
	;
	db $42+8	; 6 more bytes are repeated for right panel lookup
	db $56+8
	db $67+8
	db $43+8
	db $55+8
	;
	db $42+8
	db $56+8
	
gem_flash
	db $50,$60,$70,$60,$50,$40,$30,$40
	
gem_chars_lo
	db gem_chars_0 & 255
	db gem_chars_1 & 255
	db gem_chars_2 & 255
	db gem_chars_3 & 255
	db gem_chars_4 & 255
	
gem_chars_hi
	db gem_chars_0 >> 8
	db gem_chars_1 >> 8
	db gem_chars_2 >> 8
	db gem_chars_3 >> 8
	db gem_chars_4 >> 8
	
gem_chars_0
	db $40,$41,$42,$43
	db $60,$61,$62,$63
	
gem_chars_1
	db $44,$45,$46,$47 
	db $64,$65,$66,$67
	
gem_chars_2
	db $48,$49,$4A,$4B
	db $68,$69,$6A,$6B
	
gem_chars_3
	db $4C,$4D,$4E,$4F
	db $6C,$6D,$6E,$6F
	
gem_chars_4
	db $50,$51,$52,$53
	db $70,$71,$72,$73
	
times_5
	db 0, 5, 10, 15, 20
	
times_5_plus_1
	db 1, 6, 11, 16, 21
	
gem_screen_pos_1_lo
	db ($0C08+0*40) & 255
	db ($0C08+1*40) & 255
	db ($0C08+2*40) & 255
	db ($0C08+3*40) & 255
	db ($0C08+4*40) & 255
	db ($0C08+5*40) & 255
	db ($0C08+6*40) & 255
	db ($0C08+7*40) & 255
	db ($0C08+8*40) & 255
	db ($0C08+9*40) & 255
	db ($0C08+10*40) & 255
	db ($0C08+11*40) & 255
	db ($0C08+12*40) & 255
	db ($0C08+13*40) & 255
	db ($0C08+14*40) & 255
	db ($0C08+15*40) & 255
	db ($0C08+16*40) & 255
	db ($0C08+17*40) & 255
	db ($0C08+18*40) & 255
	db ($0C08+19*40) & 255
	db ($0C08+20*40) & 255
	db ($0C08+21*40) & 255
	
gem_screen_pos_1_hi
	db ($0C08+0*40) >> 8
	db ($0C08+1*40) >> 8
	db ($0C08+2*40) >> 8
	db ($0C08+3*40) >> 8
	db ($0C08+4*40) >> 8
	db ($0C08+5*40) >> 8
	db ($0C08+6*40) >> 8
	db ($0C08+7*40) >> 8
	db ($0C08+8*40) >> 8
	db ($0C08+9*40) >> 8
	db ($0C08+10*40) >> 8
	db ($0C08+11*40) >> 8
	db ($0C08+12*40) >> 8
	db ($0C08+13*40) >> 8
	db ($0C08+14*40) >> 8
	db ($0C08+15*40) >> 8
	db ($0C08+16*40) >> 8
	db ($0C08+17*40) >> 8
	db ($0C08+18*40) >> 8
	db ($0C08+19*40) >> 8
	db ($0C08+20*40) >> 8
	db ($0C08+21*40) >> 8
	
	; must be aligned
lightning_1
	db $00,$00,$00,$00,$00,$C0,$40,$C0,$00,$00,$00,$00,$00
lightning_2
	db $00,$00,$00,$00,$00,$30,$10,$30,$00,$00,$00,$00,$00
lightning_3
	db $00,$00,$00,$00,$00,$0C,$04,$0C,$00,$00,$00,$00,$00
lightning_4
	db $00,$00,$00,$00,$00,$03,$01,$03,$00,$00,$00,$00,$00

;panel_left
;	db $81,$82,$83,$82,$83,$82,$84
;	db $85,$98,$98,$98,$98,$98,$85
;	db $85,$98,$98,$98,$98,$98,$85
;	db $85,$98,$98,$98,$98,$98,$85
;	db $86,$98,$98,$98,$98,$98,$86
;	
;	db $87,$82,$83,$82,$83,$82,$88
;	db $86,$20,$20,$20,$20,$20,$86
;	db $85,$92,$93,$92,$93,$92,$85
;	db $86,$20,$20,$20,$20,$20,$86
;	db $87,$82,$83,$82,$83,$82,$88
;	
;	db $86,$20,$20,$20,$20,$20,$86
;	db $87,$83,$84,$20,$81,$82,$8A
;	db $85,$20,$85,$20,$85,$20,$20
;	db $85,$20,$89,$82,$88,$20,$20
;	db $85,$20,$20,$20,$85,$20,$20
;
;	db $85,$20,$20,$20,$89,$82,$84
;	db $85,$20,$20,$20,$20,$20,$85
;	db $86,$20,$20,$20,$20,$20,$86
;	db $87,$82,$82,$83,$8C,$83,$88
;	db $85,$99,$9A,$9B,$85,$20,$85
;	
;	db $85,$99,$9A,$9B,$85,$20,$86
;	db $85,$99,$9A,$9B,$85,$20,$80
;	db $85,$9C,$9D,$9E,$85,$20,$86
;	db $85,$20,$94,$95,$88,$20,$85
;	db $89,$83,$96,$97,$89,$82,$8A
;	
;panel_right
;	db $81,$82,$83,$82,$83,$82,$84
;	db $85,$C0,$C1,$C2,$C3,$C4,$85
;	db $85,$C5,$C6,$C7,$C8,$C9,$85
;	db $87,$82,$83,$82,$83,$82,$88
;	db $86,$20,$20,$20,$20,$20,$86
;	
;	db $85,$20,$8F,$90,$91,$20,$85
;	db $85,$20,$8F,$90,$91,$20,$85
;	db $87,$8D,$8F,$90,$91,$8E,$88
;	db $85,$20,$8F,$90,$91,$20,$85
;	db $85,$20,$8F,$90,$91,$20,$85
;	
;	db $86,$20,$20,$20,$20,$20,$86
;	db $87,$82,$83,$82,$83,$82,$88
;	db $86,$20,$20,$20,$20,$20,$86
;	db $85,$92,$93,$92,$93,$92,$85
;	db $85,$20,$20,$20,$20,$20,$85
;	
;	db $89,$82,$83,$8C,$83,$82,$88
;	db $20,$20,$20,$85,$20,$20,$85
;	db $20,$94,$95,$CA,$83,$82,$88
;	db $81,$96,$97,$20,$20,$20,$85
;	db $CB,$CC,$CD,$20,$81,$82,$88
;	
;	db $86,$20,$CE,$CF,$D0,$20,$85
;	db $80,$20,$20,$20,$85,$20,$85
;	db $86,$20,$20,$81,$8A,$20,$85
;	db $85,$20,$20,$86,$20,$20,$85
;	db $89,$82,$83,$8B,$83,$82,$8A
	
times_32_lo
	db (0*32) & 255
	db (1*32) & 255
	db (2*32) & 255
	db (3*32) & 255
	db (4*32) & 255
	db (5*32) & 255
	db (6*32) & 255
	db (7*32) & 255
	db (8*32) & 255
	db (9*32) & 255

times_32_hi
	db (0*32) >> 8
	db (1*32) >> 8
	db (2*32) >> 8
	db (3*32) >> 8
	db (4*32) >> 8
	db (5*32) >> 8
	db (6*32) >> 8
	db (7*32) >> 8
	db (8*32) >> 8
	db (9*32) >> 8
	
times_48_lo
	db (0*48) & 255
	db (1*48) & 255
	db (2*48) & 255
	db (3*48) & 255
	db (4*48) & 255
	db (5*48) & 255
	db (6*48) & 255
	db (7*48) & 255
	db (8*48) & 255
	db (9*48) & 255

times_48_hi
	db (0*48) >> 8
	db (1*48) >> 8
	db (2*48) >> 8
	db (3*48) >> 8
	db (4*48) >> 8
	db (5*48) >> 8
	db (6*48) >> 8
	db (7*48) >> 8
	db (8*48) >> 8
	db (9*48) >> 8
	
main_menu_screen_pos_lo
	db (MAIN_MENU_SCREEN_POS+0*40) & 255
	db (MAIN_MENU_SCREEN_POS+1*40) & 255
	db (MAIN_MENU_SCREEN_POS+2*40) & 255
	
main_menu_screen_pos_hi
	db (MAIN_MENU_SCREEN_POS+0*40) >> 8
	db (MAIN_MENU_SCREEN_POS+1*40) >> 8
	db (MAIN_MENU_SCREEN_POS+2*40) >> 8
	
title_flash_colors
	db $66+8, $76+8, $71+8, $76+8
	
	cmap "A",1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26
txt_title
	db $A0,$A1,$A2,$A3,$A4,$A3,$A5,$A6,$A7,$A8,$A1,$A2,$A9,$A4,$A3,$AA
	db $AB,$AC,$AC,$AD,$AB,$AD,$AB,$AE,$AF,$AC,$AC,$AC,$B0,$AB,$AD,$B1
	db $B2,$B3,$B4,$B5,$B6,$B7,$B2,$B8,$B9,$BA,$BA,$BB,$BC,$B6,$BD,$BE
txt_menu
	db " START GAME "
	db "INSTRUCTIONS"
	db "  CREDITS   "
	
txt_game_over
	db "GAME"
	db "OVER"
	
txt_congrats
	db "CONGRATULATIONS, NEW"
txt_high_score
	db "HIGH SCORE:  000"
	
txt_instructions
	db $04,"DEFIANCE: A PUZZLE SHOOTER"
	db $01
	db $01
	db $06,"MOVE YOUR SHIP LEFT/RIGHT,"
	db $06,"SHOOT THE FALLING BLOCKS  "
	db $06,"WITH THE CORRECT COLOR.   "
	db $06,"USE UP/DOWN TO            "
	db $06,"CHANGE COLOR.             "
	db $01
	db $01
	db $03,"THE GAME SPEED CONSTANTLY "
	db $03,"INCREASES.                "
	db $01
	db $01
	db $05,"SHOOTING SAME COLORED     "
	db $05,"BLOCKS IN A ROW SCORES    "
	db $05,"MORE POINTS, AND SHOOTING "
	db $05,"4 IN A ROW REDUCES SPEED. "
	db $01
	db $01
	db $07,"GOOD LUCK!                "
	
txt_credits
	db $04,"DEFIANCE: A PUZZLE SHOOTER"
	db $01
	db $01
	db $01
	db $06,"      CODE,               "
	db $06,"      MUSIC,              "
	db $06,"      GRAPHICS BY:        "
	db $06,"      CSABO OF LOD        "
	db $01
	db $01
	db $03,"      WRITTEN IN          "
	db $03,"      DECEMBER 2021       "
	db $01
	db $01
	db $05,"      THANKS TO           "
	db $05,"      - CHRONOS           "
	db $05,"      - GERLICZER         "
	db $05,"      - KICHY             "
	db $05,"      - LUCA              "
	db $05,"      - SIZ               "
	db $05,"        FOR TESTING!      "
	
to_hex
	db "0123456789ABCDEF"
	cmap
	
	include "gfx.asm"

	include "music/!player.asm"
	include "music/!player_data.asm"
	include "music/music.asm"
;eof