	;
	; Universal Keyboard handler loop
	; 
	; Prerequisites:
	;
	; keyb_ptr = $##	; zeropage variable (16 bits)
	; keyb_lastkey = $##	; zeropage variable (8 bits)
	; keyb_repeatcnt = $##	; zeropage variable (8 bits)
	; keyb_shift = $##	; zeropage variable (8 bits)
	;
	; Input: A/X: pointer to a keyboard table
	;
	;   Structure of the table:
	;
	;   2 bytes:
	;     - pointer to an 'interrupt' subroutine that will be called each frame 
	;
	;   4 bytes each
	;     - 1 byte: key code column
	;               or $01/$02 for joystick
	;               or $00 for end of key table
	;     - 1 byte: key code row
	;     - 2 bytes: jump address
	;
keyboard
	cmp keyb_ptr
	bne keyb_x
	cpx keyb_ptr+1
	beq keyb_s			; both A/X were the same, so don't reset anything, just continue
	;
keyb_x	sta keyb_ptr			; store pointer
	stx keyb_ptr+1
	;
	ldy #$00			; first two bytes in the table: "interrupt"
	lda (keyb_ptr),y		; any subroutine that will be called each frame
	sta keyb_i+1    
	iny
	lda (keyb_ptr),y
	sta keyb_i+2
	;
keyb_0	ldx #$FF
	stx keyb_lastkey		; clear last pressed keycode
	;
	inx
	stx keyb_repeatcnt		; reset repeat count to zero
	;
keyb_s	
	;lda #$CC			; default code removed due to digi
	;cmp $FF1D
	;bne *-3
	jsr sync
	;
keyb_i	jsr *
	;
	lda #$FD			; check Shift
	;
	sta $FD30
	sta $FF08
	lda $FF08
	;
	and #$80
	eor #$80
	sta keyb_shift			; set shift flag
	;
keyb_ns	;
	;
	ldx #$00			; X is the index counter
	ldy #$02			; Y is the pointer in the table
	;
keyb_1	lda (keyb_ptr),y		; keyboard check
	beq keyb_0			; 0 means end of table, nothing is pressed, restart checking
	cmp #$07
	bcs keyb_2			; >7 means keyboard
	;
	eor #$FF			; turn $01 into $FE
	bne keyb_j			; always jump
	;
keyb_2	sta $FD30
keyb_j	sta $FF08
	lda $FF08
	;
	iny
	and (keyb_ptr),y
	beq keyb_p			; something is pressed!
	;
	iny				; skip this byte and 2 more bytes
	iny
	iny
	inx				; next key
	bne keyb_1			; always jump
	;
keyb_p	
	lda #keyb_repeat_init		; initial repeat counter
	;
	cpx keyb_lastkey		; is this the same as last time?
	bne keyb_newkey	
	;  
	dec keyb_repeatcnt		; same key is being held down, decrement repeat counter
	lda keyb_repeatcnt		; did we reach zero?
	bne keyb_s			; no, so don't do anything this time, restart checking
	;
	lda #keyb_repeat		; continuous repeat counter
	;
keyb_newkey
	sta keyb_repeatcnt
	;
	stx keyb_lastkey		; store this key as "last key" for next round
	;               
	iny
	lda (keyb_ptr),y
	sta keyb_jump+1
	iny
	lda (keyb_ptr),y
	sta keyb_jump+2
	;
keyb_jump
	jmp *				; self-mod, make the jump (X will contain the key index)
;[eof]